#include "cipher.h"

nzCipherText::nzCipherText() {
  reset();
  name = "unnamed";
}
nzCipherText::nzCipherText(string tName) {
  reset();
  name = tName;
}
nzCipherText::nzCipherText(string tText, string tName) {
  reset();
  name = tName;
  add(tText);  
}

void nzCipherText::reset() {
  text = "";
  for (int k=0; k<26; k++) freq[k]=0;
}

void nzCipherText::addFile(const char* fName) {
  ifstream inFile(fName, ios::in);
  string strIn = "";
  string buffer = "";

  if (inFile.fail()) {
    cerr << "Error: input file " << fName << " doesn't exist." << endl;
    exit(1);
  }

  while (inFile.good()) {
    inFile >> buffer;
    if (inFile.good()) strIn += buffer;
  }
  inFile.close();
  add(strIn);
}

void nzCipherText::add(char addChar) {
  text += addChar;
  freq[(int)addChar - (int)'a']++;
}
void nzCipherText::add(int charCode) {
  text += char(charCode + (int)'a');
  freq[charCode]++;
}
void nzCipherText::add(string strIn) {
  string stripped = stripText(strIn);
  text += stripped;
  for (int k=0; k<stripped.length(); k++) {
    freq[(int)stripped[k] - (int)'a']++;
  }
}

void nzCipherText::blockPrint() {
  if (text.length()==0) {
    cout << "\n" << name << " is empty." << endl;
    return;
  }

  cout << "\n" << name << " (" << text.length() << " characters, " 
       << identify() << "):" << endl;
  for (int k=0; k<text.length(); k++) {
    if (k % 5 == 0 && k>0) cout << " ";
    if (k %40 == 0 && k>0) cout << "\n";
    cout << char((int)(text[k])+(int)'A'-(int)'a');
  }
  cout << endl;
  cout << setprecision(4);
  cout << " chi2-en: " << getEnglishChi2() 
       << " chi2-fr: " << getFrenchChi2() 
       << " chi2-unif: " << getUniformChi2()        
       << " chi2-refl: " << getReflectorChi2() << endl;
}

double nzCipherText::getEnglishChi2() {
  double enFreq[] = {.0808, .0167, .0318, .0399, .1256, .0217, .0180, .0527, .0724,
		     .0014, .0063, .0404, .0260, .0738, .0747, .0191, .0009, .0642,
		     .0659, .0915, .0279, .0100, .0189, .0021, .0165, .0007};
  double chi2 = 0;
  double length = (double)(text.length());
  for (int k=0; k<26; k++) {
    double chi = freq[k]-length*enFreq[k];
    chi2 += chi*chi/(length*enFreq[k]);
  }
  return chi2/26.;
}
double nzCipherText::getFrenchChi2() {
  double frFreq[] = {.0942, .0102, .0264, .0339, .1587, .0095, .0104, .0077, .0841, 
		     .0089, .0001, .0534, .0324, .0715, .0514, .0286, .0106, .0646, 
		     .0790, .0726, .0624, .0215, .0001, .0030, .0024, .0032};
  double chi2 = 0;
  double length = (double)(text.length());
  for (int k=0; k<26; k++) {
    double chi = freq[k]-length*frFreq[k];
    chi2 += chi*chi/(length*frFreq[k]);
  }
  return chi2/26.;
}
double nzCipherText::getUniformChi2() {
  double unFreq = 1./26.;
  double chi2 = 0;
  double length = (double)(text.length());
  for (int k=0; k<26; k++) {
    double chi = freq[k]-length*unFreq;
    chi2 += chi*chi/(length*unFreq);
  }
  return chi2/26.;
}
double nzCipherText::getReflectorChi2() {
  double rfFreq[] = {0.03672, 0.0394, 0.03888, 0.03828, 0.03496, 0.03912, 0.0392, 0.03756, 0.0372, 
		     0.03992, 0.03968, 0.0384, 0.03904, 0.03732, 0.0370, 0.03924, 0.03996, 0.0376, 
		     0.03748, 0.03636, 0.03888, 0.0396, 0.03908, 0.03996, 0.0392, 0.03996};
  double chi2 = 0;
  double length = (double)(text.length());
  for (int k=0; k<26; k++) {
    double chi = freq[k]-length*rfFreq[k];
    chi2 += chi*chi/(length*rfFreq[k]);
  }
  return chi2/26.;
}

string nzCipherText::stripText(string strIn) {
  string strOut = "";
  for (int k=0; k<strIn.length(); k++) {
    int test = (int)(strIn[k]);
    if (test>=(int)'a' && test<=(int)'z') strOut += char(test);
    if (test>=(int)'A' && test<=(int)'Z') strOut += char(test-(int)'A'+(int)'a');
  }
  return strOut;
}

string nzCipherText::identify() {
  vector<double> vChi2;
  vector<string> vId;
  vChi2.push_back(getEnglishChi2());   vId.push_back("English");
  vChi2.push_back(getFrenchChi2());    vId.push_back("French");
  vChi2.push_back(getUniformChi2());   vId.push_back("random");
  vChi2.push_back(getReflectorChi2()); vId.push_back("reflector-encrypted");

  return vId[getMinIndex(vChi2)];
}

int nzCipherText::getMinIndex(vector<double> vect) {
  int minIndex=0;
  for (int k=1; k<vect.size(); k++) if (vect[k]<vect[minIndex]) minIndex=k;
  return minIndex;
}

void nzCipherText::writeFreqTable(const char* outFile) {
  ofstream fout(outFile);
  fout << "# frequency table for " << name 
       << "\n# " << text.length() << " characters, " << identify() << endl;
  double N = (double)text.length();
  for (int k=0; k<26; k++) {
    fout << k << "\t" << char(k+(int)'a') << "\t" << freq[k]/N << endl;
  }
  fout.close();
  cout << "Wrote frequency table for " << name << " to " << outFile << endl;
}
