#ifndef NEMA_H 
#define NEMA_H

/*! \mainpage
Documentation for the simulation of the Swiss Army
<a  target="_top" href="http://lphe.epfl.ch/~nzwahlen/bonus/crypto/nema/">NeMa</a>
cipher machine (1947).
Simulation by C. Ehret and N. Zwahlen, 2007
*/

using namespace std;

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include "cipher.h"

//! Simulation of the Swiss Army NeMa cipher machine (1947)
class nzNema {
 private:
  //! positions of the 10 rotor wheels
  int state[10];
  //! number of keystrokes 
  int steps;
  //! inner key: list of notch rings
  const int* notchRings[6];
  //! inner key: list of contact wheels
  const int* rotors[4];
  //! inner key: list of contact wheels (for backwards flow)  
  const int* invrot[4];
  //! container for encrypted text
  nzCipherText cipher;
  //! current flow from keyboard to lamps, through the wheels
  char flow(char charIn);
  //! step the rotor wheels after a keystroke
  void stepState();
  //! show current 10-letter password
  void printState();
  //! set inner key (contact wheels and notch rings, from right to left!)
  void setInnerKey();
  //! relative positions of two adjacent contact wheels
  int getWheelOffset(int wheel);

 public:
  //! default constructor
  nzNema();
  //! constructor with 10-letter password
  nzNema(string passwd);
  //! encrypt or decrypt a string
  string crypt(string strIn);
  //! encrypt or decrypt a ciphertext string
  string crypt(nzCipherText ctext) {return crypt(ctext.str());};
  //! fast (no output) encryption of a string. Returns english-chi2.
  double fastCrypt(string strIn);
  //! fast (no output) encryption of a ciphertext string
  double fastCrypt(nzCipherText ctext) {return fastCrypt(ctext.str());};
  //! known-plaintext attack, returns true if key is correct (set key first)
  bool kpta(const string strPlain, const string strCipher);
  //! set 10-letter password passing array of alphabetical codes [0-25]
  void initState(int init[10]);
  //! set 10-letter password passing a string
  void initState(string passwd);
  //! return uniform-random-chi2 of ciphertext
  double getUniformChi2() {return cipher.getUniformChi2();};
};



#endif

